/***************************************************************************
 *   Copyright (C) 2007-2015 by Helge Kortendieck                          *
 *   alan@kasmithee.de                                                     *
 *   settings.cpp  -  set the settings/config options                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 *                                                                         *
 *   siimvi - settings dialog                                              * 
 ***************************************************************************/
#include "settings.h"

setDlg::setDlg(QDialog* parent, Qt::WindowFlags fl)
	: QDialog(parent, fl)
{
	uis.setupUi(&wSetDlg);
	setRead();
}

setDlg::~setDlg()
{
}


void setDlg::setRead()
{
	QSettings settings("siimvi", "siimvi");

	settings.beginGroup("main");
		pos = settings.value("pos", QPoint(0, 0)).toPoint();										// settings dialog position and size
		size = settings.value("size", QSize(330, 550)).toSize();
		uis.cB_ShowFitToScreen->setChecked(settings.value("showfittoscreen", "true").toBool());		// 0=orig size, 1=fit to screen
		uis.cB_ShowName->setChecked(settings.value("showname", "false").toBool());					// show image name on image open, 0=no, 1=yes
		uis.cB_ShowSize->setChecked(settings.value("showsize", "false").toBool());					// show image size on image open, 0=no, 1=yes
		uis.spB_ShowSeconds->setValue(settings.value("showseconds", 3).toInt());					// time (sec) to show name and/or size on image open, default = 3 sec
		uis.spB_ShowFontSize->setValue(settings.value("showfontsize", 12).toInt());					// font size (points) to show name and/or size on image open, default = 12
		uis.cB_AutoRotateOnExif->setChecked(settings.value("autorotateonexif", "false").toBool());	// use exif info to auto-rotate image on image open, 0=no, 1=yes
		uis.cB_InfoFont->setChecked(settings.value("usediffinfofont", "false").toBool());			// use a different font size for info widget
		uis.spB_InfoFont->setValue(settings.value("infowidgetfontsize", 11).toInt());				// font size for info widget
	settings.endGroup();

	connect(uis.cB_ShowFitToScreen, SIGNAL(stateChanged(int)), this, SLOT(dlgEnableSave()));
	connect(uis.cB_ShowName, SIGNAL(stateChanged(int)), this, SLOT(dlgEnableSave()));
	connect(uis.cB_ShowSize, SIGNAL(stateChanged(int)), this, SLOT(dlgEnableSave()));
	connect(uis.spB_ShowSeconds, SIGNAL(valueChanged(int)), this, SLOT(dlgEnableSave()));
	connect(uis.spB_ShowFontSize, SIGNAL(valueChanged(int)), this, SLOT(dlgEnableSave()));
	connect(uis.cB_AutoRotateOnExif, SIGNAL(stateChanged(int)), this, SLOT(dlgEnableSave()));
	connect(uis.cB_InfoFont, SIGNAL(stateChanged(int)), this, SLOT(dlgEnableSave()));
	connect(uis.spB_InfoFont, SIGNAL(valueChanged(int)), this, SLOT(dlgEnableSave()));
	connect(uis.pB_Cancel, SIGNAL(clicked()), this, SLOT(dlgClose()));
	connect(uis.pB_Save, SIGNAL(clicked()), this, SLOT(dlgSave()));

	wSetDlg.resize(size);
	wSetDlg.move(pos);

	bsD = false;
	wSetDlg.show();
}


void setDlg::setWrite()
{
	QSettings settings("siimvi", "siimvi");

	settings.beginGroup("main");
		settings.setValue("showfittoscreen", uis.cB_ShowFitToScreen->isChecked());		// 0=orig size, 1=fit to screen
		settings.setValue("showname", uis.cB_ShowName->isChecked());					// show image name on image open, 0=no, 1=yes
		settings.setValue("showsize", uis.cB_ShowSize->isChecked());					// show image size on image open, 0=no, 1=yes
		settings.setValue("showseconds", uis.spB_ShowSeconds->value());					// time (sec) to show name and/or size on image open, default = 3 sec
		settings.setValue("showfontsize", uis.spB_ShowFontSize->value());				// font size (points) to show name and/or size on image open, default = 12
		settings.setValue("autorotateonexif", uis.cB_AutoRotateOnExif->isChecked());	// use exif info to auto-rotate image on image open, 0=no, 1=yes
		settings.setValue("usediffinfofont", uis.cB_InfoFont->isChecked());				// use a different font size for info widget
		settings.setValue("infowidgetfontsize", uis.spB_InfoFont->value());				// font size for info widget
	settings.endGroup();

	settings.sync();
}

void setDlg::setWriteCancel()
{
	QSettings settings("siimvi", "siimvi");

	settings.beginGroup("main");
		settings.setValue("pos", wSetDlg.pos());
		settings.setValue("size", wSetDlg.size());
	settings.endGroup();

	settings.sync();
}

void setDlg::dlgEnableSave()                   // enable save button
{
	bsD = true;
	uis.pB_Save->setEnabled(true);
}

void setDlg::dlgSave()                      // save settings
{
	setWrite();
	dlgClose();
}

void setDlg::dlgClose()                     // close settings dialog
{
	setWriteCancel();
	emit sigReReadSet(bsD);
	wSetDlg.hide();
}


//#include "settings.moc"
