/****************************************************************************
 **
 **  Copyright (C) 2007-2019 by Helge Kortendieck, alan@kasmithee.de
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 or later as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#ifndef _SIV_H_
#define _SIV_H_

#include <QAction>
#include <QApplication>
#include <QCursor>
#include <QDesktopWidget>
#include <QDialog>
#include <QDir>
#include <QEvent>
#include <QFileDialog>
#include <QFileInfo>
#include <QFont>
#include <QImage>
#include <QImageReader>
#include <QKeyEvent>
#include <QLabel>
#include <QList>
#include <QMatrix>
#include <QMenu>
#include <QMessageBox>
#include <QMouseEvent>
#include <QPainter>
#include <QPen>
#include <QPixmap>
#include <QPoint>
#include <QRect>
#include <QScreen>
#include <QSettings>
#include <QString>
#include <QStringList>
#include <QTextCodec>
#include <QTimer>
#include <QToolTip>
#include <QWheelEvent>
#include <QWidget>
#include <QWindow>
#include <QX11Info>

#include "abouthelp.h"
#include "settings.h"
#include "tt.h"
//#include "question.h"
#include "info.h"

#include <exiv2/exiv2.hpp>
//#include <exiv2/image.hpp>
//#include <exiv2/exif.hpp>

#include "ui_info.h"

#include <iostream>
#include <cassert>
//#include <iomanip>
//#include <string>
//#include <stdexcept>
using namespace std;

class Siv : public QWidget        //MainWindow
{
	Q_OBJECT

public:
	Siv(QWidget *parent = 0);
		~Siv();

protected:

protected slots:
	void keyPressEvent(QKeyEvent*);
	void mousePressEvent(QMouseEvent*);
	void mouseMoveEvent(QMouseEvent*);
	void wheelEvent(QWheelEvent*);


	private:
	Ui::Info uii;			// image information widget

	bool bAnimated;			// animated image, shown via QMovie
	bool bBlockWheel;		// block selecting next image until current is loaded or timed out (3 sec)
	bool bD;
	bool bExif;				// use exif info to auto-rotate image on image open acc settings, 0=no, 1=yes
	bool bExifTool;			// ExifTool available, 0=no, 1=yes
	bool bFiles;			// got file or files? Look for others in dir or cycle through given files with mouse wheel up/down
	bool bFTS;				// current image view: 0=1:1, 1=fit to screen
	bool bGrab;				// current grab status: 0=no, 1=yes
	bool bGrabX;			// image is wider than screen in width, need to grab image
	bool bGrabY;			// image is wider than screen in heigth, need to grab image
	bool bInit;				// starting up, first show image, then setup rest of application
	bool bLoadNext;			// loading of image failed, try to load another image after telling user
	bool bQuit;				// quit when receive signal that message box was hidden
	bool bShowName;			// current show image name on image open: 0=no, 1=yes
	bool bShowSize;			// current show image size on image open: 0=no, 1=yes

	bool bWAhw;				// abouthelp widget created ?
	bool bWInfo;			// info widget created ?
	bool bWQuest;			// question widget created?
	bool bWSetDlg;			// settings dialog created ?

	double dD;				// dummy double
	double dD2;				// dummy double
	double dX;				// used for scaling an image
	double dY;				// used for scaling an image

	int iAnimatedDelay;		// delay in ms between animated images
	int iAnimatedImgCur;	// current image of animated image
	int iAnimatedImgMax;	// number of images in animated image
	int iC;					// current cursor image: 0=arrow, 1=wait, 2=closed hand, 3=blank
	int iCurFN;				// index of current image when starting load (actual iCurImage may differ due to quick move to pre-loaded prev/next image
	int iCPX;				// cursor position x
	int iCPY;				// cursor position y
	int iD;					// dummy int
	int iD2;				// dummy int
	int iErr;				// error code from loading image
	int iImCurListIdx;		// index of currently shown image in file list slFiles
	int iImCurX;			// width of image currently to show
	int iImCurY;			// heigth of image currently to show
	int iImWidgetUpLeftX;	// x-coordinate of top left corner where pixmap is shown in widget
	int iImWidgetUpRightY;	// y-coordinate of top left corner where pixmap is shown in widget
	int iImOrigX;			// original width of current image
	int iImOrigY;			// original heigth of current image
	int iImLastGood;		// last file from file list that we could successfully load
	int iImMoveX;			// image move in x (mouse move * factor)
	int iImMoveY;			// image move in y (mouse move * factor)
	int iImShownX;			// width of image to be shown after any manipulation
	int iImShownY;			// heigth of image to be shown after any manipulation
	int iLastVisCur;		// last visible setted cursor type, 0=arrow, 1=wait, 2=closed hand, 3=blank
	int iLoad;				// tracker what we are loading: 0=idle, not loading image, 1=current, 2=next, 3=previous image
	int iMFactX;			// mouse move factor x, move image x*mouse move
	int iMFactY;			// mouse move factor y, move image y*mouse move
	int iMnX;				// new mouse x pos in grab mode
	int iMnY;				// new mouse y pos in grab mode
	int iMoX;				// old mouse x pos in grab mode
	int iMoY;				// old mouse y pos in grab mode
	int iMMoveX;			// mouse move in x
	int iMMoveY;			// mouse move in y
	int iMLimLowX;			// lower mouse cursor position limit in x
	int iMLimUpX;			// upper mouse cursor position limit in x
	int iMLimLowY;			// lower mouse cursor position limit in y
	int iMLimUpY;			// upper mouse cursor position limit in y
	int iImErr;				// error code while trying to load image
	int iImStartX;			// top left x screen coordinate of the image part to draw
	int iImStartY;			// top left y screen coordinate of the image part to draw
	int iMaxOffX;			// maximum allowed X offset ( image width - screen width if image w > screen w )
	int iMaxOffY;			// maximum allowed Y offset ( image height - screen height if image h > screen h )

	int iRotate;			// image rotation base from it's original orientation: 0=as is, 1=90° right, 2=180° right, 3=90° left

	int iScrX;				// screen width
	int iScrY;				// screen heigth
	int iScrX3;				// width of the middle 9th box, the area to move the mouse to move the image in X
	int iScrY3;				// height of the middle 9th box, the area to move the mouse to move the image in Y
	int iScrStartX;			// top left x screen coordinate to start drawing the image
	int iScrStartY;			// top left y screen coordinate to start drawing the image
	int iWToShow;			// popup widget to show: 0=none, 1=about-abouthelp, 2=help-abouthelp, 3=error-abouthelp, 4=info, 5=settings dialog
	int iWShown;			// popup widget currently shown: 0=none, 1=about-abouthelp, 2=help-abouthelp, 3=error-abouthelp, 4=info, 5=settings dialog

	uint uiD;
	uint uiImD;				// dummy uint

	QAction *actInfo;		// info, i-key, action for menu to show image information
	QAction *actInfoClose;	// close image info, action for menu to close image information widget
	QAction *actLeft;		// left, l-key, action for menu to rotate image 90° left
	QAction *actRight;		// right, r-key, action for menu to rotate image 90° right
	QAction *actView0;		// view, v-key, action for menu to show image 1:1
	QAction *actView1;		// view, v-key, action for menu to show image maximized to screen
	QAction *actName;		// name, n-key, action for menu to show image name (for x sec defined in settings dialog)
	QAction *actSize;		// size, s-key, action for menu to show image size (for x sec defined in settings dialog)
	QAction *actSaveImg;	// save image, m-key, action to save current image
	QAction *actDlg;		// dialog (settings), d-key, action for menu to open settings dialog
	QAction *actHelp;		// help, h-key, action for menu to open help dialog
	QAction *actAbout;		// about, a-key, action for menu to open about dialog
	QAction *actQuit;		// quit, q-key, action for menu to exit program

	QByteArray qba;

	QChar qchr;

	QDialog wInfo;			// image info widget

	QImage qiCur;			// current image, as loaded
	QImage qiCurC;			// copy of current image, as loaded, for saving

	QImageReader imReader;	// image reader

	QMenu *menu;			// mini menu accessible by right-clicking into the image

	QPixmap qpmCur;			// image shown in the widget

	QPoint qpD;				// dummy qpoint
	QPoint qpMenu;			// show delete menu at this point
	QPoint qPos;			// cursor position

	QRect qrFrame;			// frame geometry = screen size

	QString sD;				// dummy QString
	QString sD2;			// dummy QString
	QString sD3;			// dummy QString
	QString sDir;			// dir to save images to
	QString sDisp;			// display to show image on open acc settings, default = 0:0
	QString sErr;			// error text when error while importing image
	QString sExOrient;		// exif tag: original image orientation
	QString sExV;			// exif tag: Exif version
	QString sFileBaseName;	// file name only (complete base name) of file to save
	QString sFileName;		// file name, base name + suffix, of file to save
	QString sFileSaveAs;	// file name incl. full path to save current picture as
	QString sFileSuffix;	// file extension only of file to save
	QString sImCur;			// image to show, full path and name
	QString sImCurFile;		// only file name of current image
	QString sImgType;		// image type when saving
	QString sMBTit;			// message titel in message widget
	QString sMBTxt;			// message text in message widget
	QString sTt;			// tooltip that we are at begin or end of image list

	QStringList slArgs;		// list with command line arguments
	QStringList slD;		// dummy QStringList
	QStringList slFiles;	// list with files/images to show
	QStringList slFilters;	// dir filter to only catch image files
	QStringList slImOri;	// list with text to describe currently shown image orientation

	QTimer *timerHideCur;	// 2 sec timer to hide mouse cursor after last move
	QTimer *timerAnimNext;  // timer in ms to show next of an animated image
	
	aboutHelp* ahw;			// help widget
	setDlg* setd;			// settings dialog
	tT* stt;				// tooltip to show at begin/end of image list
	tT* sns;				// tooltip to show image name and/or size
	InfoW* inf;				// info widget with file and exif information of current image
//	quest* que;				// question widget
	
private slots:
	void about();				// show about information
	void help();				// show help message
	void ahwCreate();			// create about/help/error dialog widget
	void ahwHidden();			// not showing the ahw widget at the moment, delete it

	void quitPrep();			// quit - clicked clean up
	void quitDo();				//      - finally quit the program

	void defActions();			// actions - define all possible actions/menu entries

	QPoint getCursorPos();		// get cursor position - iD=x , iD2=y

	void imAlign();				// adjust image to center of screen or set crop to show (if image larger then screen)

	void imInfoClose();										// info widget	- close image information popup window
	void imInfoErr(const QString& sD, const QString& sD2);	//				- error while getting exif tags for info widget - aborting
	void imInfoHidden();									//				- image information widget was hidden
	void imInfoShow();										//				- show image information in popup window
	void imInfoToFront();									//				- bring info widget back to front
	void imInfoUpd();										//				- update image information

	void imLoad(const QString& sD);	// try loading image	- sD, full path and name
	void imLoadErr();				//						- error while trying

	void imResize();				// need to resize image, either to fit-to-screen or back to original-size

	void imRotateGetExifHint();		// check Exif info if need to rotate, not set here, set in next step
	void imRotate();				// rotate image according exif orientation correction and user changes

	void imShow();					// show image
	void imShowAnimNext();          // show next image of an animated image

	void init();					// set everything up (nearly), to allow a fast start, do after showing first image !

	void menuToggleFitToScreen();	// menu clicked	- show image fit to screen in menu selected, toggle view
	void menuLeft();				//				- rotate image 90° to left
	void menuPop();					//				- populate the menu
	void menuRight();				//				- rotate image 90° to right

	int messageBox(int iD, const QString& sD, const QString& sD2);	// show QMessageBox: type 1=info, 2=warn, 3=critical, 4=question
																	// at current cursor position
																	// on question returns 5=yes or 6=no

	void paintEvent(QPaintEvent*);

//	void questCreate();					// question dialog widget - create
//	void questHidden( bool bD );		//                         - not showing at the moment, delete it

	void readDir();						// get all images in dir of image called
	void removeImCurFromList();			// remove current image from image list, failed to load
	void requestedNewImage(int iD);		// user requested new image -1=prev or +1=next

	void saveImg();						// save current image
	void saveImgSetNameType();			//						- set image name and type for file to save
	void saveImgGetFileName();			//						- get file name and path
	void saveImgSave();					//						- save do

	void setDlgShow();					// settings dialog	- show
	void setDlgClosed(bool bD);			//					- was closed, bool whether re-read settings (if canceled, then not)

	void setCurGrab();					// set cursor	- grab mode
	void setCurPos();					//				- position
	void setCurType(int iC);			//				- type (image), 0=arrow, 1=wait, 2=closed hand

	void setRead();
	void setWrite();

	void showImNameSize();				// show name and/or size of image a short time

	void timerAnimNextCreate();         // create timer to show next image of an animated image
//    void timerAnimNextDestroy();        // needed?
	void timerHideCurCreate();			// 2 sec timer to hide mouse cursor	- create (hide cursor in grab mode after last move)
	void timerHideCurStart();			//									- start
	void timerHideCurStop();			//									- stop
	void timerHideCurTimedOut();		//									- timed out

	void ttRec(const QString& sD);	 	// tooltip	- received from other program parts
	void ttShow();						//			- show at begin or end of list

	void unBlockWheelTimer();			// start 2,7 sec single shot timer to unblock the mouse wheel in case loading another image failed
	void unBlockWheel();				// unblock the mouse wheel to allow moving to prev/next image in case loading the other image takes more than 2.7 sec (failed?)

	void usrImageNew(const QString& sD);// show new image
	void usrImageChange();				// resize (FTS/orig size) or change rotation of current image on user request

	void wCtrl(int iWToShow);			// show popup widget, if no other is shown: 0=none, 1=about-abouthelp, 2=help-abouthelp, 3=error-abouthelp, 4=info, 5=settings dialog

	signals:
		void sigDlgAbout();										// show dialog with about text
		void sigDlgHelp();										// show dialog with help text
		void sigDlgMsg(const QString&, const QString&);			// show dialog with message text
		void sigInfGet(const QString&, int, int, int, int, int);// get image information, fill info widget
		void sigInfoToFrt();									// bring info dlg back to frt
		void sigInfoHide();
		void sigQuestAsk(int, const QString&);
		void sigSetShow();										// show settings dialog
		void sigShowListBeginEnd(QPoint , const QString&);		// show tooltip at begin/end of image list
		void sigSetReadTt();									// tooltip for info at begin/end list shall re-read settings
		void sigShowNameSize(int, int, const QString&);			// show name and/or size for int seconds
		void sigSetReadTtNS();									// tooltip showing name and/or size on image load shall re-read settings
};


#endif 
