/***************************************************************************
 *   Copyright (C) 2007-2009 by Helge Kortendieck  alan@kasmithee.de       *
 *   tt.cpp  -  show tooltip, not influenced by Qt4 events                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/
#include "tt.h"

tT::tT( QWidget* parent, Qt::WindowFlags fl )
        : QWidget( parent, fl )
{
  setStyleSheet("background-color: rgb( 232, 232, 164 )");
  setWindowOpacity( 0.75 );
  QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);

  l_Tt = new QLabel(this);
    l_Tt->setAutoFillBackground( false );
    l_Tt->setText( "" );
    l_Tt->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);

  setRead();

  timerNS = new QTimer(this);               // show name and/or size timer
  timerNS->setSingleShot( true );
  connect(timerNS, SIGNAL(timeout()), this, SLOT( tClose() ) );

  timerTt = new QTimer(this);               // show tooltip at begin/end of image list
  timerTt->setSingleShot( true );
  connect(timerTt, SIGNAL(timeout()), this, SLOT( tClose() ) );
}

tT::~tT()
{
}

void tT::setRead()
{
  QSettings settings("siimvi", "siimvi");

  settings.beginGroup( "main" );
    iTime = settings.value( "showseconds", 3 ).toInt();       // time (sec) to show name and/or size on image open, default = 3 sec
    iFont = settings.value( "showfontsize", 12 ).toInt();     // font size (points) to show name and/or size on image open, default = 12
  settings.endGroup();
}

void tT::tShow( QPoint pos, const QString& sTt )  // show tooltip at begin/end of image list
{
  bShow = true;
  if( timerTt->isActive() == true ) {
    timerTt->stop();
  }
  l_Tt->setStyleSheet("font-size: " + QString::number(iFont) + "pt");
  l_Tt->setText( sTt );
  l_Tt->adjustSize();
  move( pos );
  show();
  timerTt->start( 2000 );
}

void tT::tShowNS( int iScrX, int iScrY, const QString& sTt )  // show tooltip name and/or size of image for iTime seconds
{
  bShow = true;
  if( timerNS->isActive() == true ) {
    timerNS->stop();
  }
  l_Tt->setStyleSheet("font-size: " + QString::number(iFont) + "pt");
  l_Tt->setText( sTt );
  l_Tt->adjustSize();
  adjustSize();
  qsFrame = l_Tt->frameSize();
  move( iScrX - qsFrame.width() + 1, iScrY - qsFrame.height() + 1 );
  show();
  timerNS->start( iTime * 1000 );
}

void tT::tClose()                           // close tooltip
{
    hide();
    bShow = false;
}


//#include "tt.moc"
